#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __FOLDERS__
#include <Folders.h>
#endif

#ifndef __DEVICES__
#include <Devices.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __PLSTRINGFUNCS__
#include <PLStringFuncs.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __DIALOGUTIL__
#include <DialogUtil.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __CCONTROLPANEL__
#include "CControlPanel.h"
#endif

#ifndef __CChooserDialog__
#include "ChooserDialog.h"
#endif

#include "Common.h"

enum
{
	kOkButton = 1,
	kCancelButton,
	kLookupButton = 8,
	kLookupAsyncButton,
	kRegisterButton,
	kRegisterAsyncButton,
	kUnregisterButton,
	kUnregisterAsyncButton,
	kItemPrefDialog = 128
};

long CControlPanel::DoItemPrefs(long theValue)
{
	DialogPtr	theDialog;
	long		thePrefs = 0;
	short		itemHit;
	GrafPtr		oldPort;
	
	GetPort(&oldPort);
	thePrefs = theValue;
	theDialog = GetNewDialog(kItemPrefDialog, nil, (WindowPtr) -1);
	SetPort(theDialog);
	TextFont(geneva);
	TextSize(9);
	if (lookupEnabled(thePrefs)) doCheck(theDialog, kLookupButton);
	if (lookupAsync(thePrefs)) doCheck(theDialog, kLookupAsyncButton);
	if (registerEnabled(thePrefs)) doCheck(theDialog, kRegisterButton);
	if (registerAsync(thePrefs)) doCheck(theDialog, kRegisterAsyncButton);
	if (removeEnabled(thePrefs)) doCheck(theDialog, kUnregisterButton);
	if (removeAsync(thePrefs)) doCheck(theDialog, kUnregisterAsyncButton);
	do
	{
		ModalDialog(nil, &itemHit);
		switch (itemHit)
		{
			case kOkButton:
				thePrefs = 0;
				thePrefs += getValue(theDialog, kLookupButton) ? kLookupEnabled : 0;
				thePrefs += getValue(theDialog, kLookupAsyncButton) ? kLookupAsync : 0;
				thePrefs += getValue(theDialog, kRegisterButton) ? kRegisterEnabled : 0;
				thePrefs += getValue(theDialog, kRegisterAsyncButton) ? kRegisterAsync : 0;
				thePrefs += getValue(theDialog, kUnregisterButton) ? kRemoveEnabled : 0;
				thePrefs += getValue(theDialog, kUnregisterAsyncButton) ? kRemoveAsync : 0;
				break;
			case kCancelButton:
				break;
			case kLookupButton:
			case kRegisterButton:
			case kUnregisterButton:
				doCheck(theDialog, itemHit);
				{
					short	theValue;
					theValue = getValue(theDialog, itemHit);
					if (theValue) setValue(theDialog, itemHit + 1, 1);
					else setValue(theDialog, itemHit + 1, 0);
				}
				break;
			default:
				doCheck(theDialog, itemHit);
				break;
		}
	} while ((itemHit != kOkButton) && (itemHit != kCancelButton));
	DisposeDialog(theDialog);
	SetPort(oldPort);
	return thePrefs;
}


long CControlPanel::DispatchMessage(short theMessage)
{
	long result = (long) this;
	switch (theMessage)
	{
		case initDev:			// init dialog
			result = DoInit();
			break;
		case hitDev:			// item hit
			result = DoHit();
			break;
		case closeDev:			// cdev closed
			result = DoClose();
			break;
		case nulDev:			// null event
			break;
		case updateDev:			// update stuff !updated by dialog
			result = DoUpdate();
			break;
		case activDev:			// activating
			result = DoActivate();
			break;
		case deactivDev:		// deactivating
			result = DoDeactivate();
			break;
		case keyEvtDev:			// keydown
			result = DoKey();
			break;
		case undoDev:			// undo chosen
		case cutDev:			// cut chosen
		case copyDev:			// copy chosen
		case pasteDev:			// paste chosen
		case clearDev:			// clear chosen
		default:
			break;
	}
	return result;
}

#pragma segment Main
CControlPanel::CControlPanel(short numItems)
{
	fNumItems = numItems;
}


#pragma segment Main
CControlPanel::~CControlPanel()
{
	
}


#pragma segment Main
void	CControlPanel::SetArguments(short item, EventRecord *theEvent, DialogPtr theDialog)
{
	fTheEvent = theEvent;
	fItemHit = item - fNumItems;
	fTheDialog = theDialog;
}


#pragma segment Main
long	CControlPanel::DoInit()
{
	long	result = (long) this;
	
	fTheList = new CListManager(kListBox, fTheDialog);
	if (fTheList) 
	{
		char	temp[5];
		
		LAddColumn(1, 2, fTheList->GetList());	// adds the column for the prefs info
		result = ReadPreferencesIntoList();
		temp[0] = 4;
		temp[1] = ((char *)&fGestaltSelector)[0] != 0 ? ((char *)&fGestaltSelector)[0] : ' ';
		temp[2] = ((char *)&fGestaltSelector)[1] != 0 ? ((char *)&fGestaltSelector)[1] : ' ';
		temp[3] = ((char *)&fGestaltSelector)[2] != 0 ? ((char *)&fGestaltSelector)[2] : ' ';
		temp[4] = ((char *)&fGestaltSelector)[3] != 0 ? ((char *)&fGestaltSelector)[3] : ' ';
		setString(fTheDialog, kGestaltSelector, (StringPtr) temp);
	}
	return result;
}

#pragma segment Main
long	CControlPanel::DoHit()
{
	long	result = (long) this;
	
	switch (fItemHit)
	{
		case kAddButton:
		{
			CChooserDialog	*theDialog;
			
			theDialog = new CChooserDialog();
			if (theDialog)
			{
				theDialog->Initialize(kChooseDialog);
				theDialog->SetChosenList(fTheList);
				theDialog->Run();
				delete theDialog;
			}
			break;
		}
		case kListBox:
			fTheList->Click(fTheEvent->where, fTheEvent->modifiers);
			hiliteItem(fTheDialog, kRemoveButton, fTheList->IsSelection() ? 0 : 255);
			hiliteItem(fTheDialog, kEditButton, fTheList->IsSelection() ? 0 : 255);
			break;
		case kRemoveButton:
			fTheList->DeleteSelection();
			hiliteItem(fTheDialog, kRemoveButton, 255);
			hiliteItem(fTheDialog, kEditButton, 255);
			break;
		case kEditButton:
			{
				Cell	theCell = {0,0};
				
				if (fTheList->FindNextSelection(theCell))
				{
					long	thePrefs;
					char	temp[5];

					theCell.h++;
					thePrefs = fTheList->GetItem((StringPtr)temp, theCell) ? *((long *)&temp[1]) : 0;
					thePrefs = DoItemPrefs(thePrefs);
					*((long *)&temp[1]) = thePrefs;
					temp[0] = 4;
					theCell.h = theCell.v = 0;
					while (fTheList->FindNextSelection(theCell))
					{
						theCell.h++;
						fTheList->SetItem((StringPtr)temp, theCell);
						theCell.h--;
						fTheList->DeselectItem(theCell);
					}
				}
			}
			break;
		default:
			break;
	}
	return result;
}


#pragma segment Main
long	CControlPanel::DoClose()
{
	SavePreferences();
	delete fTheList;
	return (long) this;
}


#pragma segment Main
long	CControlPanel::DoUpdate()
{
	DrawSmallFrame(fTheDialog, kListBox);
	fTheList->Update();
	return (long) this;
}


#pragma segment Main
long	CControlPanel::DoActivate()
{
	hiliteItem(fTheDialog, kAddButton, 0);
	hiliteItem(fTheDialog, kRemoveButton, fTheList->IsSelection() ? 0 : 255);
	hiliteItem(fTheDialog, kEditButton, fTheList->IsSelection() ? 0 : 255);
/*
	getditem gets the edittext of the item, not the itemhandle. Caused a bus error
	hiliteItem(fTheDialog, kGestaltSelector, 0);
*/
	fTheList->Activate();
	return (long) this;
}


#pragma segment Main
long	CControlPanel::DoDeactivate()
{
	fTheList->Deactivate();
	hiliteItem(fTheDialog, kAddButton, 255);
	hiliteItem(fTheDialog, kRemoveButton, 255); 	
	hiliteItem(fTheDialog, kEditButton, 255);
	/*
	getditem gets the edittext of the item, not the itemhandle. Caused a bus error
	hiliteItem(fTheDialog, kGestaltSelector, 255);
*/
	return (long)this;
}


#pragma segment Main
long	CControlPanel::DoKey()
{
	Str32	theString;
	char	key = fTheEvent->message & charCodeMask;
	
	if ((key == '*') || (key == '') || (key == '='))
	{
		SysBeep(5);
		fTheEvent->what = 0;
	}
	else
	{
		TEHandle theTERec;
		
		theTERec = ((DialogPeek)fTheDialog)->textH;	// get the TERec of the gestalt selector
		
		if (!((**theTERec).selEnd - (**theTERec).selStart)) // if theres no selection
		{
			getString(fTheDialog, kGestaltSelector, theString);
			if ((theString[0] == 4) && (key != 0x08))
			{
				SysBeep(5);
				fTheEvent->what = nullEvent;
			}
		}
	}
	return (long) this;
}

#pragma segment Main



#pragma segment Main
void CControlPanel::InsertItems()
{
	Handle			temp;
	Str32			rName;
	short			rID;
	OSType			rType;
	short			numTypes = Count1Resources('KILL');
	
	while (numTypes)
	{
		temp = Get1IndResource('KILL', numTypes);
		if (temp)
		{
			long	size;
			Cell	theCell = {0,0};
			
			GetResInfo(temp, &rID, &rType, rName);
			size = GetHandleSize(temp);
			if (size == sizeof(long))
			{
				size = *((long *)*temp);
			}
			else
			{
				size = kLookupEnabled + kLookupAsync;
			}
			ReleaseResource(temp);
			fTheList->InsertItem(rName);
			fTheList->FindItem(rName, theCell);
			theCell.h++;
			LSetCell(&size, sizeof(long), theCell, fTheList->GetList());
		}
		numTypes--;
	}
}

#pragma segment Main
void CControlPanel::SavePreferences()
{
	Str32	theString;
	Str32	thePrefs;
	short	refNum;
	FSSpec	theFile;
	Handle	theHandle = NewHandle(sizeof(long));
	long	result = (long) this;
	Cell	theCell;
	
	FindFolder(-1, kPreferencesFolderType, true, &theFile.vRefNum, &theFile.parID);
	GetIndString(theString, kPrefsName, 0);
	if (!theString[0]) PLstrcpy(theFile.name, "\pIncognito Prefs");
	else PLstrcpy(theFile.name, theString);
	refNum = FSpOpenResFile(&theFile, fsRdWrPerm);
	if (refNum == -1) return;
	
	RemovePreferences();
	theCell.v = 0;
	theCell.h = 0;
	if (fTheList->GetFirstItem(theString, theCell))
	do
	{
		theCell.h += 1;
		*((long *)*theHandle) =	fTheList->GetItem(thePrefs, theCell) ?
						*(long *)(&thePrefs[1]) :
						kLookupEnabled + kLookupAsync;
		theCell.h -= 1;
		AddResource(theHandle, 'KILL', theCell.v, theString);
		WriteResource(theHandle);
		DetachResource(theHandle);
	} while (fTheList->GetNextItem(theString, theCell));
	DisposeHandle(theHandle);
	CloseResFile(refNum);
	{
		FInfo	theInfo;
		Str32	theString;
		
		getString(fTheDialog, kGestaltSelector, theString);
		((char *)&fGestaltSelector)[0] = theString[1] != 0 ? theString[1] : ' ';
		((char *)&fGestaltSelector)[1] = theString[2] != 0 ? theString[2] : ' ';
		((char *)&fGestaltSelector)[2] = theString[3] != 0 ? theString[3] : ' ';
		((char *)&fGestaltSelector)[3] = theString[4] != 0 ? theString[4] : ' ';
		FSpGetFInfo(&theFile, &theInfo);
		theInfo.fdCreator = fGestaltSelector;
		FSpSetFInfo(&theFile, &theInfo);
	}
}

#pragma segment Main
void CControlPanel::RemovePreferences()
{
	Handle			temp;
	Str32			rName;
	short			rID;
	OSType			rType;
	short			numTypes = Count1Resources('KILL');
	
	while (numTypes)
	{
		temp = Get1IndResource('KILL', numTypes);
		if (temp)
		{
			RemoveResource(temp);
		}
		numTypes--;
	}
	UpdateResFile(CurResFile());
}


#pragma segment Main
long CControlPanel::ReadPreferencesIntoList(void)
{
	Str255	theString;
	short	refNum;
	FSSpec	theFile;
	long	result = (long) this;
	
	FindFolder(-1, kPreferencesFolderType, true, &theFile.vRefNum, &theFile.parID);
	GetIndString(theString, kPrefsName, 1);
	if (!theString[0])
	{
		PLstrcpy(theFile.name, "\pIncognito Prefs");
	}
	PLstrcpy(theFile.name, theString);
	refNum = FSpOpenResFile(&theFile, fsRdPerm);
	if (refNum == -1)
	{
		fGestaltSelector = kDefaultSelector;
		FSpCreateResFile(&theFile, fGestaltSelector, 'pref', 0);
		refNum = FSpOpenResFile(&theFile, fsRdPerm);
	}
	else
	{
		FInfo	theInfo;
		
		FSpGetFInfo(&theFile, &theInfo);
		fGestaltSelector = theInfo.fdCreator;
	}
	if (refNum != -1)
	{
		InsertItems();
		CloseResFile(refNum);
	} else result = cdevResErr;
	return result;
}




